--[[
Smoke is always transitioned to outside theme
]]
function smokebg(image, position)
  --[[
    position: 0 or nil -- center, 1 -- bottom-left
  ]]
  local realscale = diretheme.get_scale();
  local ws, hs = diretheme.get_scaled_screen();
  local w = ws * realscale;
  local h = hs * realscale;
  
  local full = pixels.new(w, h);
  -- Background
  local bg = pixels.new('images/main_theme/background.png'):scale(realscale);
  dire_draw_tile(bg, full, {x=0, y=0, w=w, h=h}, {xoffset = -1, yoffset = -1});
  
  -- Load Image
  local pic = pixels.new(image);
  local px, py, ps = pic:size();
  local bdx = math.floor((ws - px) / 2);
  local bdy = math.floor((hs - py) / 2);
  if position == 1 then
    if bdx < 0 then
      bdx = 0;
    end;
    if bdy < 0 then
      bdy = hs - py;
    end;
  end;
  -- Frame
  local tbg, tbg_slice = make_tbg_slice(realscale, 'main');
  local bgframe, fdx, fdy, frw, frh = make_ending_frame(realscale, 'main', tbg, tbg_slice);
  bgframe:copy(full, bdx * realscale - fdx, bdy * realscale - fdy);

  -- Draw Image
  pic:scale(realscale):copy(full, bdx * realscale, bdy * realscale);
  
  return full
end;

function coords(x)
  return x, math.floor(x / 480 * 300);
end;

function enableSmoke(image, position)
  local bg = smokebg(image, position);
  
  local w, h = diretheme.get_screen();
  local animscale = math.max(math.ceil(w / 960), math.ceil(h / 600));
  
  local adx = math.floor(w - animscale * 960) / 2;
  local ady = math.floor(h - animscale * 600) / 2;
  
  local smoke_data = {
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 3994, 319, 51, 49, 0, 551};
      };
    }; -- 0
    {
      trans = {0, 551, 51, 49};
      images = {
        {1, 3566, 1425, 69, 125, 0, 0},
        {1, 892, 1518, 67, 81, 893, 0},
        {1, 3961, 1014, 77, 64, 0, 536},
        {1, 3975, 703, 95, 48, 865, 552},
      };
    }; -- 1
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2753, 1457, 93, 81, 0, 0},
        {1, 2043, 1525, 113, 52, 847, 548},
        {1, 3951, 128, 98, 59, 862, 0},
        {1, 882, 1214, 75, 76, 0, 524},
        {1, 2170, 1525, 29, 47, 0, 100},
        {1, 3423, 1578, 33, 27, 927, 59},
        {1, 1074, 1589, 30, 16, 926, 86},
        {1, 2320, 1591, 46, 10, 396, 590},
        {1, 4055, 1234, 19, 24, 0, 296},
        {1, 2738, 1228, 6, 3, 1, 292},
      };
    }; -- 2
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2747, 840, 127, 91, 833, 0},
        {1, 3914, 1141, 81, 93, 0, 38},
        {1, 1920, 1525, 123, 49, 837, 534},
        {1, 3975, 415, 74, 76, 0, 496},
        {1, 2880, 1559, 125, 38, 0, 0},
        {1, 465, 1574, 101, 28, 0, 572},
        {1, 3840, 992, 156, 17, 804, 583},
        {1, 2645, 1564, 44, 41, 0, 291},
        {1, 1574, 1549, 54, 31, 0, 131},
        {1, 1151, 1473, 45, 29, 915, 91},
        {1, 4004, 101, 66, 19, 384, 581},
        {1, 2091, 1382, 65, 11, 403, 0},
        {1, 4049, 151, 21, 17, 917, 120}
      }
    }; -- 3
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2735, 1232, 143, 119, 817, 0},
        {1, 3840, 0, 144, 73, 0, 0},
        {1, 2441, 1425, 90, 97, 0, 470},
        {1, 2091, 1232, 100, 83, 0, 73},
        {1, 2204, 1564, 166, 27, 794, 573},
        {1, 2814, 1376, 54, 80, 0, 280},
        {1, 227, 1560, 124, 33, 0, 567},
        {1, 3840, 804, 132, 31, 828, 542},
        {1, 2533, 1564, 112, 35, 375, 565},
        {1, 3975, 491, 95, 34, 865, 119},
        {1, 4024, 1262, 39, 78, 921, 243},
        {1, 0, 1574, 92, 22, 394, 0},
        {1, 2441, 1522, 74, 25, 875, 517},
        {1, 3986, 877, 72, 20, 0, 156},
        {1, 264, 1594, 69, 10, 884, 153},
        {1, 2720, 1527, 33, 19, 864, 164},
        {1, 2191, 1232, 10, 16, 139, 75},
        {1, 1438, 1511, 4, 6, 488, 594},
        {1, 1441, 1556, 1, 4, 0, 274},
      };
    }; -- 4
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 1920, 1232, 171, 162, 789, 0},
        {1, 361, 1362, 172, 117, 0, 0},
        {1, 3837, 1353, 122, 93, 0, 484},
        {1, 882, 1101, 77, 113, 883, 239},
        {1, 3840, 1083, 163, 50, 797, 550},
        {1, 3840, 472, 133, 57, 0, 117},
        {1, 361, 1527, 104, 55, 856, 495},
        {1, 1444, 1549, 130, 43, 9, 441},
        {1, 3977, 260, 80, 59, 0, 269},
        {1, 3856, 1559, 99, 39, 372, 540},
        {1, 3964, 1563, 110, 33, 380, 0},
        {1, 2043, 1577, 150, 23, 0, 577},
        {1, 960, 1568, 160, 21, 337, 579},
        {1, 2406, 1504, 35, 44, 0, 328},
        {1, 1780, 1560, 119, 11, 8, 174},
        {1, 1589, 1518, 41, 23, 0, 372},
      }
    }; -- 5
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 1444, 1200, 187, 189, 0, 0},
        {1, 3777, 1262, 171, 85, 789, 70},
        {1, 960, 1473, 191, 60, 769, 540},
        {1, 3840, 128, 111, 80, 849, 292},
        {1, 3840, 1014, 121, 69, 839, 471},
        {1, 3840, 600, 168, 47, 0, 553},
        {1, 3840, 647, 141, 56, 339, 526},
        {1, 3840, 370, 174, 45, 786, 0},
        {1, 3840, 897, 142, 52, 0, 457},
        {1, 3194, 1296, 108, 65, 0, 275},
        {1, 227, 1505, 122, 55, 377, 0},
        {1, 1631, 1572, 202, 25, 758, 45},
        {1, 2558, 1202, 194, 18, 323, 582},
        {1, 4007, 949, 55, 58, 0, 340},
        {1, 2752, 1202, 106, 30, 0, 509},
        {1, 2370, 1591, 142, 14, 0, 539},
        {1, 3566, 1579, 74, 26, 0, 398},
        {1, 4004, 73, 50, 28, 910, 264},
        {1, 4053, 33, 22, 28, 0, 247},
        {1, 2156, 1382, 40, 12, 920, 155},
        {1, 4070, 120, 6, 56, 0, 190},
        {1, 1438, 1495, 6, 16, 415, 509},
        {1, 4070, 751, 4, 2, 956, 261},
        {1, 713, 1603, 2, 1, 518, 599},
      }
    }; -- 6
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 1208, 1200, 236, 167, 0, 0},
        {1, 2532, 1232, 203, 144, 757, 0},
        {1, 1444, 1389, 178, 97, 0, 451},
        {1, 3091, 1441, 207, 75, 753, 525},
        {1, 1631, 1481, 198, 52, 0, 548},
        {1, 2747, 931, 130, 66, 0, 282},
        {1, 3840, 415, 135, 57, 825, 468},
        {1, 3840, 703, 135, 56, 825, 311},
        {1, 3840, 759, 166, 45, 325, 535},
        {1, 960, 1533, 161, 35, 368, 0},
        {1, 1208, 1556, 143, 39, 0, 167},
        {1, 3457, 1540, 108, 47, 852, 144},
        {1, 3959, 1353, 88, 57, 0, 348},
        {1, 3981, 647, 95, 47, 413, 35},
        {1, 3840, 579, 213, 20, 312, 580},
        {1, 3988, 529, 74, 43, 371, 492},
        {1, 1920, 1574, 91, 29, 0, 422},
        {1, 3005, 1559, 57, 46, 0, 236},
        {1, 766, 1568, 84, 30, 876, 281},
        {1, 533, 1362, 46, 48, 914, 233},
        {1, 3211, 1582, 93, 17, 0, 405},
        {1, 2204, 1591, 116, 13, 3, 206},
        {1, 4004, 234, 67, 20, 893, 367},
        {1, 3194, 1361, 71, 11, 883, 387},
        {1, 427, 1582, 33, 20, 915, 191},
        {1, 852, 1384, 21, 29, 154, 422},
        {1, 196, 1574, 24, 20, 936, 448},
        {1, 4049, 168, 16, 19, 403, 472},
        {1, 4062, 1007, 14, 14, 596, 586},
        {1, 4062, 1021, 13, 12, 462, 82},
      };
    }; -- 7
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 3307, 1141, 232, 212, 0, 0},
        {1, 1208, 1367, 226, 96, 734, 504},
        {1, 960, 1387, 235, 86, 725, 65},
        {1, 3643, 1353, 194, 103, 0, 427},
        {1, 2532, 1457, 221, 70, 0, 530},
        {1, 1208, 1463, 199, 65, 761, 0},
        {1, 3307, 1501, 244, 39, 286, 561},
        {1, 3840, 73, 164, 55, 796, 305},
        {1, 3840, 837, 146, 60, 0, 289},
        {1, 3840, 260, 137, 59, 348, 502},
        {1, 2880, 1517, 183, 42, 356, 0},
        {1, 3840, 529, 148, 50, 812, 454},
        {1, 2091, 1315, 97, 67, 863, 238},
        {1, 2707, 1551, 130, 43, 830, 151},
        {1, 1121, 1533, 80, 62, 0, 227},
        {1, 3959, 1410, 106, 43, 0, 349},
        {1, 3984, 33, 69, 40, 423, 63},
        {1, 713, 1557, 51, 44, 295, 517},
        {1, 3566, 1550, 77, 29, 883, 360},
        {1, 3307, 1578, 116, 19, 383, 42},
        {1, 3240, 1408, 57, 32, 365, 470},
        {1, 3211, 1559, 78, 23, 0, 404},
        {1, 850, 1568, 37, 35, 923, 419},
        {1, 1589, 1486, 39, 32, 575, 568},
        {1, 152, 1594, 112, 10, 0, 392},
        {1, 2809, 1351, 57, 18, 903, 194},
        {1, 1151, 1517, 53, 14, 907, 389},
        {1, 4049, 128, 18, 23, 169, 404},
        {1, 2406, 1548, 184, 2, 0, 402},
        {1, 4058, 877, 18, 13, 446, 103},
        {1, 713, 1601, 78, 2, 414, 61},
        {1, 2531, 1536, 1, 1, 826, 446},
      };
    }; -- 8
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 3539, 1141, 238, 201, 0, 399},
        {1, 960, 1200, 248, 187, 0, 0},
        {1, 297, 1238, 278, 118, 682, 482},
        {1, 581, 1384, 271, 81, 689, 78},
        {1, 3307, 1425, 259, 76, 276, 524},
        {1, 3777, 1141, 137, 121, 0, 278},
        {1, 3643, 1456, 228, 70, 732, 8},
        {1, 361, 1479, 215, 48, 337, 0},
        {1, 2204, 1504, 202, 47, 758, 297},
        {1, 3871, 1456, 93, 92, 351, 432},
        {1, 3840, 319, 154, 51, 338, 48},
        {1, 581, 1518, 185, 39, 775, 443},
        {1, 766, 1518, 126, 50, 834, 247},
        {1, 3307, 1540, 150, 38, 810, 159},
        {1, 3643, 1559, 115, 41, 845, 344},
        {1, 4006, 759, 67, 65, 0, 213},
        {1, 2370, 1564, 163, 26, 0, 187},
        {1, 4003, 1150, 65, 45, 555, 555},
        {1, 3539, 1342, 234, 8, 726, 0},
        {1, 3240, 1374, 55, 34, 905, 409},
        {1, 2735, 1351, 74, 15, 886, 197},
        {1, 3972, 804, 34, 25, 315, 499},
        {1, 1574, 1580, 50, 16, 910, 385},
        {1, 3063, 1517, 26, 22, 140, 336},
        {1, 4057, 260, 19, 27, 148, 213},
        {1, 2187, 1475, 16, 30, 445, 494},
        {1, 1899, 1560, 18, 12, 443, 99},
        {1, 1201, 1533, 7, 9, 553, 0},
        {1, 3295, 1559, 9, 6, 776, 437},
      };
    }; -- 9
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2204, 840, 543, 164, 0, 436},
        {1, 0, 1238, 297, 124, 663, 476},
        {1, 1631, 1200, 258, 127, 0, 0},
        {1, 581, 1299, 297, 85, 663, 76},
        {1, 2204, 1337, 273, 88, 0, 127},
        {1, 2204, 1425, 237, 79, 723, 272},
        {1, 0, 1429, 246, 76, 714, 0},
        {1, 2880, 1441, 211, 76, 327, 0},
        {1, 1920, 1475, 250, 50, 0, 386},
        {1, 3840, 208, 164, 52, 0, 269},
        {1, 246, 1429, 102, 75, 557, 525},
        {1, 3840, 949, 167, 43, 793, 161},
        {1, 3643, 1526, 213, 33, 747, 443},
        {1, 3948, 1262, 76, 86, 354, 350},
        {1, 0, 1545, 196, 29, 0, 321},
        {1, 3063, 1559, 148, 36, 0, 350},
        {1, 3914, 1234, 141, 23, 819, 351},
        {1, 3984, 0, 86, 33, 874, 239},
        {1, 4008, 600, 60, 43, 900, 382},
        {1, 4003, 1195, 67, 38, 411, 76},
        {1, 1833, 1572, 78, 32, 0, 223},
        {1, 960, 1589, 114, 14, 0, 255},
        {1, 4049, 415, 24, 65, 170, 256},
        {1, 2558, 1220, 180, 8, 0, 215},
        {1, 361, 1582, 66, 18, 745, 425},
        {1, 1407, 1463, 32, 32, 237, 354},
        {1, 1407, 1495, 31, 32, 158, 223},
        {1, 4027, 1540, 49, 18, 911, 425},
        {1, 1151, 1502, 50, 15, 910, 204},
        {1, 3837, 1446, 93, 8, 867, 374},
        {1, 4047, 1367, 23, 13, 447, 114},
        {1, 333, 1594, 18, 11, 664, 161},
      },
    }; -- 10
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2880, 1014, 960, 127, 0, 473},
        {1, 1920, 840, 284, 392, 0, 81},
        {1, 2204, 1232, 328, 105, 632, 81},
        {1, 1631, 1327, 284, 81, 676, 0},
        {1, 2532, 1376, 282, 81, 0, 0},
        {1, 1920, 1394, 267, 81, 301, 0},
        {1, 581, 1465, 292, 53, 668, 247},
        {1, 1444, 1486, 145, 63, 295, 410},
        {1, 0, 1505, 227, 40, 733, 433},
        {1, 3091, 1516, 204, 43, 756, 300},
        {1, 1208, 1528, 233, 28, 727, 405},
        {1, 581, 1557, 132, 42, 828, 343},
        {1, 465, 1527, 106, 47, 380, 81},
        {1, 2532, 1527, 188, 21, 772, 186},
        {1, 4027, 1456, 43, 84, 379, 299},
        {1, 3986, 837, 75, 40, 640, 433},
        {1, 1351, 1556, 82, 30, 878, 207},
        {1, 4004, 208, 70, 26, 356, 384},
        {1, 2707, 1594, 171, 10, 789, 237},
        {1, 2846, 1538, 34, 46, 441, 427},
        {1, 882, 1497, 72, 20, 888, 385},
        {1, 1780, 1533, 49, 27, 442, 128},
        {1, 92, 1574, 60, 20, 722, 385},
        {1, 2477, 1337, 40, 18, 775, 207},
        {1, 4053, 579, 20, 21, 631, 186},
        {1, 4047, 1353, 28, 14, 462, 155},
        {1, 2515, 1522, 16, 14, 717, 371},
        {1, 3840, 835, 86, 1, 336, 383},
        {1, 4063, 1262, 13, 6, 284, 0},
        {1, 2406, 1550, 1, 1, 378, 296},
      }
    }; -- 11
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2880, 840, 960, 174, 0, 426},
        {1, 2204, 1004, 354, 228, 606, 0},
        {1, 0, 1101, 581, 137, 0, 0},
        {1, 2558, 1004, 316, 198, 644, 228},
        {1, 581, 1101, 301, 198, 0, 228},
        {1, 2880, 1296, 314, 78, 0, 137},
        {1, 882, 1299, 43, 198, 600, 228},
        {1, 3964, 1456, 63, 107, 364, 280},
        {1, 1829, 1481, 86, 78, 427, 137},
        {1, 2204, 1551, 503, 13, 0, 215},
        {1, 1631, 1533, 149, 39, 315, 387},
        {1, 3986, 897, 84, 51, 305, 228},
        {1, 882, 1290, 17, 6, 435, 381},
      };
    }; -- 12
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2880, 600, 960, 237, 0, 0},
        {1, 0, 893, 960, 208, 0, 392},
        {1, 2880, 1141, 427, 155, 0, 237},
        {1, 3307, 1353, 336, 72, 624, 237},
        {1, 1631, 1408, 276, 73, 684, 309},
        {1, 1208, 1595, 360, 10, 600, 382},
        {1, 4045, 319, 28, 73, 603, 309},
        {1, 1195, 1387, 13, 30, 597, 248},
      };
    }; -- 13
    {
      trans = {83, 0, 865, 600};
      images = {
        {1, 0, 600, 960, 293, 0, 307},
        {1, 1920, 600, 960, 240, 0, 0},
        {1, 0, 1362, 361, 67, 0, 240},
        {1, 2880, 1374, 360, 67, 600, 240},
        {1, 4003, 1083, 54, 67, 367, 240},
        {1, 2188, 1315, 15, 32, 579, 253},
      };
    }; -- 14
    {
      trans = {49, 0, 896, 600};
      images = {
        {1, 0, 0, 960, 600, 0, 0}
      };
    }; -- 15
    {
      trans = {70, 0, 849, 600};
      images = {
        {1, 960, 0, 960, 600, 0, 0}
      }
    }; -- 16
    {
      trans = {97, 0, 750, 537};
      images = {
        {1, 1920, 0, 960, 600, 0, 0}
      }
    }; -- 17
    {
      trans = {145, 13, 661, 490};
      images = {
        {1, 2880, 0, 960, 600, 0, 0}
      };
    }; -- 18
    {
      trans = {154, 34, 646, 437};
      images = {
        {1, 960, 600, 960, 600, 0, 0}
      };
    }; -- 19
    {
      trans = {131, 30, 632, 467};
      images = {
        {2, 0, 0, 960, 600, 0, 0}
      };
    }; -- 20
    {
      trans = {104, 105, 629, 371};
      images = {
        {2, 0, 600, 960, 600, 0, 0};
      };
    }; -- 21
    {
      trans = {179, 100, 535, 256};
      fill = {0, 193, 19, 251};
      images = {
        {2, 0, 1200, 941, 600, 19, 0}
      }
    }; -- 22
    {
      trans = {199, 128, 398, 203};
      fill = {19, 209, 941, 249};
      images = {
        {2, 0, 1800, 928, 600, 31, 0}
      }
    }; -- 23
    {
      trans = {240, 152, 372, 274};
      fill = {31, 158, 928, 367};
      images = {
        {2, 0, 2400, 784, 600, 63, 0}
      };
    }; -- 24
    {
      trans = {351, 165, 202, 205};
      fill = {63, 159, 784, 329};
      images = {
        {2, 784, 2400, 765, 600, 70, 0}
      };
    }; -- 25
    {
      trans = {247, 190, 351, 135};
      fill = {70, 90, 765, 255};
      images = {
        {2, 928, 1800, 727, 600, 77, 0}
      };
    }; -- 26
    {
      trans = {385, 280, 181, 27};
      fill = {171, 0, 633, 315};
      images = {
        {2, 767, 3000, 755, 592, 0, 8}
      };
    }; -- 27
    {
      fill = {135, 8, 620, 586};
      images = {
        {2, 960, 0, 570, 496, 166, 27}
      };
    }; -- 28
    {
      fill = {166, 27, 281, 325};
      images = {
        {2, 0, 3000, 767, 547, 193, 53}
      };
    }; -- 29
    {
      fill = {193, 151, 767, 449};
      images = {
        {2, 960, 496, 511, 501, 195, 36}
      };
    }; -- 30
    {
      fill = {195, 36, 511, 501};
      images = {
        {2, 960, 997, 429, 297, 260, 101}
      };
    }; -- 31
    {
      fill = {260, 101, 429, 297};
      images = {
        {2, 0, 3547, 400, 274, 271, 111}
      };
    }; -- 32
    {
      fill = {271, 342, 278, 43};
      images = {
        {2, 767, 3592, 400, 262, 281, 84}
      };
    }; -- 33
    {
      fill = {281, 84, 384, 251};
      images = {
        {2, 960, 1294, 421, 224, 299, 124}
      };
    }; -- 34
    {
      fill = {641, 124, 79, 28};
      images = {
        {2, 1167, 3592, 383, 236, 299, 125}
      };
    }; -- 35
    {
      fill = {677, 125, 5, 5};
      images = {
        {2, 960, 1518, 413, 234, 297, 157}
      };
    }; -- 36
    {
      fill = {297, 157, 413, 234}
    }; -- 37
    {
      images = {
        {2, 960, 1518, 413, 234, 297, 157}
      };
    }; -- 36
    {
      fill = {297, 157, 413, 234};
      images = {
        {2, 1167, 3592, 383, 236, 299, 125}
      };
    }; -- 35
    {
      fill = {299, 125, 383, 236};
      images = {
        {2, 960, 1294, 421, 224, 299, 124}
      };
    }; -- 34
    {
      fill = {299, 124, 421, 224};
      images = {
        {2, 767, 3592, 400, 262, 281, 84}
      };
    }; -- 33
    {
      fill = {281, 84, 400, 262};
      images = {
        {2, 0, 3547, 400, 274, 271, 111}
      };
    }; -- 32
    {
      fill = {271, 111, 400, 274};
      images = {
        {2, 960, 997, 429, 297, 260, 101}
      };
    }; -- 31
    {
      fill = {260, 101, 429, 297};
      images = {
        {2, 960, 496, 511, 501, 195, 36}
      };
    }; -- 30
    {
      fill = {195, 36, 511, 501};
      images = {
        {2, 0, 3000, 767, 547, 193, 53}
      };
    }; -- 29
    {
      fill = {193, 53, 767, 547};
      images = {
        {2, 960, 0, 570, 496, 166, 27}
      };
    }; -- 28
    {
      trans = {385, 280, 181, 27};
      fill = {166, 27, 570, 496};
      images = {
        {2, 767, 3000, 755, 592, 0, 8}
      };
    }; -- 27
    {
      trans = {247, 190, 351, 135};
      fill = {0, 8, 755, 592};
      images = {
        {2, 928, 1800, 727, 600, 77, 0}
      };
    }; -- 26
    {
      trans = {351, 165, 202, 205};
      fill = {77, 0, 727, 600};
      images = {
        {2, 784, 2400, 765, 600, 70, 0}
      };
    }; -- 25
    {
      trans = {240, 152, 372, 274};
      fill = {70, 0, 765, 600};
      images = {
        {2, 0, 2400, 784, 600, 63, 0}
      };
    }; -- 24
    {
      trans = {199, 128, 398, 203};
      fill = {63, 0, 784, 600};
      images = {
        {2, 0, 1800, 928, 600, 31, 0}
      }
    }; -- 23
    {
      trans = {179, 100, 535, 256};
      fill = {31, 0, 928, 600};
      images = {
        {2, 0, 1200, 941, 600, 19, 0}
      }
    }; -- 22
    {
      trans = {104, 105, 629, 371};
      images = {
        {2, 0, 600, 960, 600, 0, 0};
      };
    }; -- 21
    {
      trans = {131, 30, 632, 467};
      images = {
        {2, 0, 0, 960, 600, 0, 0}
      };
    }; -- 20
    {
      trans = {154, 34, 646, 437};
      images = {
        {1, 960, 600, 960, 600, 0, 0}
      };
    }; -- 19
    {
      trans = {145, 13, 661, 490};
      images = {
        {1, 2880, 0, 960, 600, 0, 0}
      };
    }; -- 18
    {
      trans = {97, 0, 750, 537};
      images = {
        {1, 1920, 0, 960, 600, 0, 0}
      }
    }; -- 17
    {
      trans = {70, 0, 849, 600};
      images = {
        {1, 960, 0, 960, 600, 0, 0}
      }
    }; -- 16
    {
      trans = {49, 0, 896, 600};
      images = {
        {1, 0, 0, 960, 600, 0, 0}
      };
    }; -- 15
    {
      trans = {83, 0, 865, 600};
      images = {
        {1, 0, 600, 960, 293, 0, 307},
        {1, 1920, 600, 960, 240, 0, 0},
        {1, 0, 1362, 361, 67, 0, 240},
        {1, 2880, 1374, 360, 67, 600, 240},
        {1, 4003, 1083, 54, 67, 367, 240},
        {1, 2188, 1315, 15, 32, 579, 253},
      };
    }; -- 14
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2880, 600, 960, 237, 0, 0},
        {1, 0, 893, 960, 208, 0, 392},
        {1, 2880, 1141, 427, 155, 0, 237},
        {1, 3307, 1353, 336, 72, 624, 237},
        {1, 1631, 1408, 276, 73, 684, 309},
        {1, 1208, 1595, 360, 10, 600, 382},
        {1, 4045, 319, 28, 73, 603, 309},
        {1, 1195, 1387, 13, 30, 597, 248},
      };
    }; -- 13
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2880, 840, 960, 174, 0, 426},
        {1, 2204, 1004, 354, 228, 606, 0},
        {1, 0, 1101, 581, 137, 0, 0},
        {1, 2558, 1004, 316, 198, 644, 228},
        {1, 581, 1101, 301, 198, 0, 228},
        {1, 2880, 1296, 314, 78, 0, 137},
        {1, 882, 1299, 43, 198, 600, 228},
        {1, 3964, 1456, 63, 107, 364, 280},
        {1, 1829, 1481, 86, 78, 427, 137},
        {1, 2204, 1551, 503, 13, 0, 215},
        {1, 1631, 1533, 149, 39, 315, 387},
        {1, 3986, 897, 84, 51, 305, 228},
        {1, 882, 1290, 17, 6, 435, 381},
      };
    }; -- 12
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2880, 1014, 960, 127, 0, 473},
        {1, 1920, 840, 284, 392, 0, 81},
        {1, 2204, 1232, 328, 105, 632, 81},
        {1, 1631, 1327, 284, 81, 676, 0},
        {1, 2532, 1376, 282, 81, 0, 0},
        {1, 1920, 1394, 267, 81, 301, 0},
        {1, 581, 1465, 292, 53, 668, 247},
        {1, 1444, 1486, 145, 63, 295, 410},
        {1, 0, 1505, 227, 40, 733, 433},
        {1, 3091, 1516, 204, 43, 756, 300},
        {1, 1208, 1528, 233, 28, 727, 405},
        {1, 581, 1557, 132, 42, 828, 343},
        {1, 465, 1527, 106, 47, 380, 81},
        {1, 2532, 1527, 188, 21, 772, 186},
        {1, 4027, 1456, 43, 84, 379, 299},
        {1, 3986, 837, 75, 40, 640, 433},
        {1, 1351, 1556, 82, 30, 878, 207},
        {1, 4004, 208, 70, 26, 356, 384},
        {1, 2707, 1594, 171, 10, 789, 237},
        {1, 2846, 1538, 34, 46, 441, 427},
        {1, 882, 1497, 72, 20, 888, 385},
        {1, 1780, 1533, 49, 27, 442, 128},
        {1, 92, 1574, 60, 20, 722, 385},
        {1, 2477, 1337, 40, 18, 775, 207},
        {1, 4053, 579, 20, 21, 631, 186},
        {1, 4047, 1353, 28, 14, 462, 155},
        {1, 2515, 1522, 16, 14, 717, 371},
        {1, 3840, 835, 86, 1, 336, 383},
        {1, 4063, 1262, 13, 6, 284, 0},
        {1, 2406, 1550, 1, 1, 378, 296},
      }
    }; -- 11
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2204, 840, 543, 164, 0, 436},
        {1, 0, 1238, 297, 124, 663, 476},
        {1, 1631, 1200, 258, 127, 0, 0},
        {1, 581, 1299, 297, 85, 663, 76},
        {1, 2204, 1337, 273, 88, 0, 127},
        {1, 2204, 1425, 237, 79, 723, 272},
        {1, 0, 1429, 246, 76, 714, 0},
        {1, 2880, 1441, 211, 76, 327, 0},
        {1, 1920, 1475, 250, 50, 0, 386},
        {1, 3840, 208, 164, 52, 0, 269},
        {1, 246, 1429, 102, 75, 557, 525},
        {1, 3840, 949, 167, 43, 793, 161},
        {1, 3643, 1526, 213, 33, 747, 443},
        {1, 3948, 1262, 76, 86, 354, 350},
        {1, 0, 1545, 196, 29, 0, 321},
        {1, 3063, 1559, 148, 36, 0, 350},
        {1, 3914, 1234, 141, 23, 819, 351},
        {1, 3984, 0, 86, 33, 874, 239},
        {1, 4008, 600, 60, 43, 900, 382},
        {1, 4003, 1195, 67, 38, 411, 76},
        {1, 1833, 1572, 78, 32, 0, 223},
        {1, 960, 1589, 114, 14, 0, 255},
        {1, 4049, 415, 24, 65, 170, 256},
        {1, 2558, 1220, 180, 8, 0, 215},
        {1, 361, 1582, 66, 18, 745, 425},
        {1, 1407, 1463, 32, 32, 237, 354},
        {1, 1407, 1495, 31, 32, 158, 223},
        {1, 4027, 1540, 49, 18, 911, 425},
        {1, 1151, 1502, 50, 15, 910, 204},
        {1, 3837, 1446, 93, 8, 867, 374},
        {1, 4047, 1367, 23, 13, 447, 114},
        {1, 333, 1594, 18, 11, 664, 161},
      },
    }; -- 10
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 3539, 1141, 238, 201, 0, 399},
        {1, 960, 1200, 248, 187, 0, 0},
        {1, 297, 1238, 278, 118, 682, 482},
        {1, 581, 1384, 271, 81, 689, 78},
        {1, 3307, 1425, 259, 76, 276, 524},
        {1, 3777, 1141, 137, 121, 0, 278},
        {1, 3643, 1456, 228, 70, 732, 8},
        {1, 361, 1479, 215, 48, 337, 0},
        {1, 2204, 1504, 202, 47, 758, 297},
        {1, 3871, 1456, 93, 92, 351, 432},
        {1, 3840, 319, 154, 51, 338, 48},
        {1, 581, 1518, 185, 39, 775, 443},
        {1, 766, 1518, 126, 50, 834, 247},
        {1, 3307, 1540, 150, 38, 810, 159},
        {1, 3643, 1559, 115, 41, 845, 344},
        {1, 4006, 759, 67, 65, 0, 213},
        {1, 2370, 1564, 163, 26, 0, 187},
        {1, 4003, 1150, 65, 45, 555, 555},
        {1, 3539, 1342, 234, 8, 726, 0},
        {1, 3240, 1374, 55, 34, 905, 409},
        {1, 2735, 1351, 74, 15, 886, 197},
        {1, 3972, 804, 34, 25, 315, 499},
        {1, 1574, 1580, 50, 16, 910, 385},
        {1, 3063, 1517, 26, 22, 140, 336},
        {1, 4057, 260, 19, 27, 148, 213},
        {1, 2187, 1475, 16, 30, 445, 494},
        {1, 1899, 1560, 18, 12, 443, 99},
        {1, 1201, 1533, 7, 9, 553, 0},
        {1, 3295, 1559, 9, 6, 776, 437},
      };
    }; -- 9
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 3307, 1141, 232, 212, 0, 0},
        {1, 1208, 1367, 226, 96, 734, 504},
        {1, 960, 1387, 235, 86, 725, 65},
        {1, 3643, 1353, 194, 103, 0, 427},
        {1, 2532, 1457, 221, 70, 0, 530},
        {1, 1208, 1463, 199, 65, 761, 0},
        {1, 3307, 1501, 244, 39, 286, 561},
        {1, 3840, 73, 164, 55, 796, 305},
        {1, 3840, 837, 146, 60, 0, 289},
        {1, 3840, 260, 137, 59, 348, 502},
        {1, 2880, 1517, 183, 42, 356, 0},
        {1, 3840, 529, 148, 50, 812, 454},
        {1, 2091, 1315, 97, 67, 863, 238},
        {1, 2707, 1551, 130, 43, 830, 151},
        {1, 1121, 1533, 80, 62, 0, 227},
        {1, 3959, 1410, 106, 43, 0, 349},
        {1, 3984, 33, 69, 40, 423, 63},
        {1, 713, 1557, 51, 44, 295, 517},
        {1, 3566, 1550, 77, 29, 883, 360},
        {1, 3307, 1578, 116, 19, 383, 42},
        {1, 3240, 1408, 57, 32, 365, 470},
        {1, 3211, 1559, 78, 23, 0, 404},
        {1, 850, 1568, 37, 35, 923, 419},
        {1, 1589, 1486, 39, 32, 575, 568},
        {1, 152, 1594, 112, 10, 0, 392},
        {1, 2809, 1351, 57, 18, 903, 194},
        {1, 1151, 1517, 53, 14, 907, 389},
        {1, 4049, 128, 18, 23, 169, 404},
        {1, 2406, 1548, 184, 2, 0, 402},
        {1, 4058, 877, 18, 13, 446, 103},
        {1, 713, 1601, 78, 2, 414, 61},
        {1, 2531, 1536, 1, 1, 826, 446},
      };
    }; -- 8
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 1208, 1200, 236, 167, 0, 0},
        {1, 2532, 1232, 203, 144, 757, 0},
        {1, 1444, 1389, 178, 97, 0, 451},
        {1, 3091, 1441, 207, 75, 753, 525},
        {1, 1631, 1481, 198, 52, 0, 548},
        {1, 2747, 931, 130, 66, 0, 282},
        {1, 3840, 415, 135, 57, 825, 468},
        {1, 3840, 703, 135, 56, 825, 311},
        {1, 3840, 759, 166, 45, 325, 535},
        {1, 960, 1533, 161, 35, 368, 0},
        {1, 1208, 1556, 143, 39, 0, 167},
        {1, 3457, 1540, 108, 47, 852, 144},
        {1, 3959, 1353, 88, 57, 0, 348},
        {1, 3981, 647, 95, 47, 413, 35},
        {1, 3840, 579, 213, 20, 312, 580},
        {1, 3988, 529, 74, 43, 371, 492},
        {1, 1920, 1574, 91, 29, 0, 422},
        {1, 3005, 1559, 57, 46, 0, 236},
        {1, 766, 1568, 84, 30, 876, 281},
        {1, 533, 1362, 46, 48, 914, 233},
        {1, 3211, 1582, 93, 17, 0, 405},
        {1, 2204, 1591, 116, 13, 3, 206},
        {1, 4004, 234, 67, 20, 893, 367},
        {1, 3194, 1361, 71, 11, 883, 387},
        {1, 427, 1582, 33, 20, 915, 191},
        {1, 852, 1384, 21, 29, 154, 422},
        {1, 196, 1574, 24, 20, 936, 448},
        {1, 4049, 168, 16, 19, 403, 472},
        {1, 4062, 1007, 14, 14, 596, 586},
        {1, 4062, 1021, 13, 12, 462, 82},
      };
    }; -- 7
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 1444, 1200, 187, 189, 0, 0},
        {1, 3777, 1262, 171, 85, 789, 70},
        {1, 960, 1473, 191, 60, 769, 540},
        {1, 3840, 128, 111, 80, 849, 292},
        {1, 3840, 1014, 121, 69, 839, 471},
        {1, 3840, 600, 168, 47, 0, 553},
        {1, 3840, 647, 141, 56, 339, 526},
        {1, 3840, 370, 174, 45, 786, 0},
        {1, 3840, 897, 142, 52, 0, 457},
        {1, 3194, 1296, 108, 65, 0, 275},
        {1, 227, 1505, 122, 55, 377, 0},
        {1, 1631, 1572, 202, 25, 758, 45},
        {1, 2558, 1202, 194, 18, 323, 582},
        {1, 4007, 949, 55, 58, 0, 340},
        {1, 2752, 1202, 106, 30, 0, 509},
        {1, 2370, 1591, 142, 14, 0, 539},
        {1, 3566, 1579, 74, 26, 0, 398},
        {1, 4004, 73, 50, 28, 910, 264},
        {1, 4053, 33, 22, 28, 0, 247},
        {1, 2156, 1382, 40, 12, 920, 155},
        {1, 4070, 120, 6, 56, 0, 190},
        {1, 1438, 1495, 6, 16, 415, 509},
        {1, 4070, 751, 4, 2, 956, 261},
        {1, 713, 1603, 2, 1, 518, 599},
      }
    }; -- 6
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 1920, 1232, 171, 162, 789, 0},
        {1, 361, 1362, 172, 117, 0, 0},
        {1, 3837, 1353, 122, 93, 0, 484},
        {1, 882, 1101, 77, 113, 883, 239},
        {1, 3840, 1083, 163, 50, 797, 550},
        {1, 3840, 472, 133, 57, 0, 117},
        {1, 361, 1527, 104, 55, 856, 495},
        {1, 1444, 1549, 130, 43, 9, 441},
        {1, 3977, 260, 80, 59, 0, 269},
        {1, 3856, 1559, 99, 39, 372, 540},
        {1, 3964, 1563, 110, 33, 380, 0},
        {1, 2043, 1577, 150, 23, 0, 577},
        {1, 960, 1568, 160, 21, 337, 579},
        {1, 2406, 1504, 35, 44, 0, 328},
        {1, 1780, 1560, 119, 11, 8, 174},
        {1, 1589, 1518, 41, 23, 0, 372},
      }
    }; -- 5
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2735, 1232, 143, 119, 817, 0},
        {1, 3840, 0, 144, 73, 0, 0},
        {1, 2441, 1425, 90, 97, 0, 470},
        {1, 2091, 1232, 100, 83, 0, 73},
        {1, 2204, 1564, 166, 27, 794, 573},
        {1, 2814, 1376, 54, 80, 0, 280},
        {1, 227, 1560, 124, 33, 0, 567},
        {1, 3840, 804, 132, 31, 828, 542},
        {1, 2533, 1564, 112, 35, 375, 565},
        {1, 3975, 491, 95, 34, 865, 119},
        {1, 4024, 1262, 39, 78, 921, 243},
        {1, 0, 1574, 92, 22, 394, 0},
        {1, 2441, 1522, 74, 25, 875, 517},
        {1, 3986, 877, 72, 20, 0, 156},
        {1, 264, 1594, 69, 10, 884, 153},
        {1, 2720, 1527, 33, 19, 864, 164},
        {1, 2191, 1232, 10, 16, 139, 75},
        {1, 1438, 1511, 4, 6, 488, 594},
        {1, 1441, 1556, 1, 4, 0, 274},
      };
    }; -- 4
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2747, 840, 127, 91, 833, 0},
        {1, 3914, 1141, 81, 93, 0, 38},
        {1, 1920, 1525, 123, 49, 837, 534},
        {1, 3975, 415, 74, 76, 0, 496},
        {1, 2880, 1559, 125, 38, 0, 0},
        {1, 465, 1574, 101, 28, 0, 572},
        {1, 3840, 992, 156, 17, 804, 583},
        {1, 2645, 1564, 44, 41, 0, 291},
        {1, 1574, 1549, 54, 31, 0, 131},
        {1, 1151, 1473, 45, 29, 915, 91},
        {1, 4004, 101, 66, 19, 384, 581},
        {1, 2091, 1382, 65, 11, 403, 0},
        {1, 4049, 151, 21, 17, 917, 120}
      }
    }; -- 3
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 2753, 1457, 93, 81, 0, 0},
        {1, 2043, 1525, 113, 52, 847, 548},
        {1, 3951, 128, 98, 59, 862, 0},
        {1, 882, 1214, 75, 76, 0, 524},
        {1, 2170, 1525, 29, 47, 0, 100},
        {1, 3423, 1578, 33, 27, 927, 59},
        {1, 1074, 1589, 30, 16, 926, 86},
        {1, 2320, 1591, 46, 10, 396, 590},
        {1, 4055, 1234, 19, 24, 0, 296},
        {1, 2738, 1228, 6, 3, 1, 292},
      };
    }; -- 2
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 3566, 1425, 69, 125, 0, 0},
        {1, 892, 1518, 67, 81, 893, 0},
        {1, 3961, 1014, 77, 64, 0, 536},
        {1, 3975, 703, 95, 48, 865, 552},
      };
    }; -- 1
    {
      trans = {0, 0, 960, 600};
      images = {
        {1, 3994, 319, 51, 49, 0, 551};
      };
    }; -- 0
  };
  
  local skip = 0;
  for index, frame in ipairs(smoke_data) do
    if frame.fill then
      if animscale ~= 1 then
        frame.fill[1] = frame.fill[1] * animscale;
        frame.fill[2] = frame.fill[2] * animscale;
        frame.fill[3] = frame.fill[3] * animscale;
        frame.fill[4] = frame.fill[4] * animscale;
      end;
      frame.fill[1] = frame.fill[1] + adx;
      frame.fill[2] = frame.fill[2] + ady;
    end;
    if frame.trans then
      if animscale ~= 1 then
        frame.trans[1] = frame.trans[1] * animscale;
        frame.trans[2] = frame.trans[2] * animscale;
        frame.trans[3] = frame.trans[3] * animscale;
        frame.trans[4] = frame.trans[4] * animscale;
      end;
      frame.trans[1] = frame.trans[1] + adx;
      frame.trans[2] = frame.trans[2] + ady;
    end;
    if frame.images then
      for _, image in ipairs(frame.images) do
        if animscale ~= 1 then
          image[2] = image[2] * animscale;
          image[3] = image[3] * animscale;
          image[4] = image[4] * animscale;
          image[5] = image[5] * animscale;
          image[6] = image[6] * animscale;
          image[7] = image[7] * animscale;
        end;
        image[6] = image[6] + adx;
        image[7] = image[7] + ady;
      end;
    end;
  end;
  
  local images = {
    pixels.new('images/endings/smoke/smoke_0.png', animscale):tosprite();
    pixels.new('images/endings/smoke/smoke_1.png', animscale):tosprite();
  }
  
  for _, image in ipairs(images) do
    image:colorkey('red');
  end;
  
  fading.change { 
    'animate', 
    animate = function(step, src, dst)      
      if step % 2 == 1 then
        step = math.floor(step / 2)
        local sd = smoke_data[step + 1];
        if sd.fill then
          sprite.scr():fill(sd.fill[1], sd.fill[2], sd.fill[3], sd.fill[4], '#b7b7b7');
        end;
        if sd.trans then
          if step < 38 then
            src:copy(sd.trans[1], sd.trans[2], sd.trans[3], sd.trans[4], sprite.scr(), sd.trans[1], sd.trans[2]);
          else
            bg:copy_spr(sd.trans[1], sd.trans[2], sd.trans[3], sd.trans[4], sprite.scr(), sd.trans[1], sd.trans[2]);
          end;
        end;
        if sd.images then
          for _, image in ipairs(sd.images) do
            images[image[1]]:copy(image[2], image[3], image[4], image[5], sprite.scr(), image[6], image[7]);
          end;
        end;  
      else
        sprite.scr():copy(0, 0, 1, 1, sprite.scr(), 0, 0);
      end; 
    end;
    max = 149
  };
  fading.enabled = true;
end;