function DSA (data)
	local scale = diretheme.get_scale();
	data[3] = get_sprite(data[3]);
	data['x'] = data['x'] * scale;
	data['y'] = data['y'] * scale;
	data['w'] = data['w'] * scale;
	data['h'] = data['h'] * scale;
	D(data);
end;

function DS (data)
	local scale = diretheme.get_scale();
	data[3] = get_sprite(data[3], 'fuchsia');
  data['background'] = true;
	data['x'] = data['x'] * scale;
	data['y'] = data['y'] * scale;
	data['w'] = data['w'] * scale;
	data['h'] = data['h'] * scale;
	D(data);
end;

local input = std.ref '@input'
function input:text(char)
  return '@text "\\' .. char .. '"';
end;

std.mod_cmd(function(cmd)
    if cmd[1] == '@text' then
      if here().ontext then
        print(cmd[2]);
        here():ontext(tostring(cmd[2]));
      end;
    end;
end)

function make_password()
  local words = {
    'Alpha';
    'Ancient';
    'Beta';
    'Biography';
    'Clear';
    'Catalyst';
    'Delta';
    'Destination';
    'Endless';
    'Esoteric';
    'Flower';
    'Freedom';
    'Gamma';
    'Geography';
    'Home';
    'History';
    'Infinite';
    'Initial';
    'Instead';
    'Journey';
    'King';
    'Knight';
    'Knowledge';
    'Lion';
    'Lost';
    'Magic';
    'Micron';
    'Night';
    'Novel';
    'Omega';
    'Ocean';
    'Precise';
    'Power';
    'Quality';
    'Question';
    'Rarity';
    'Reason';
    'Silent';
    'Secret';
    'Tiger';
    'Theta';
    'Tower';
    'Ultimate';
    'Universe';
    'Visitor';
    'Vital';
    'Wolf';
    'Wizardry';
  };
  local numbers = {'0','1','2','3','4','5','6','7','8','9'};
  
  return random_phrase(words) .. '_' .. random_phrase(words) .. '_' .. random_phrase(numbers) .. random_phrase(numbers);
end;

function getscreen(graphical)
	local d = D('terminal_screen');
	if not d or d.graphical ~= graphical then
    local scale = diretheme.get_scale();
		local sw, sh = diretheme.get_scaled_screen();
		local x = math.floor((sw - 569) / 2 - 16);
    local dy = 0;
    if sh < 540 then
      dy = 19;
    end;
		local y = math.max(sh - 736 - dy, 25 - dy);
		local h = 320;
    if sh < 500 then
      h = h - 16;
    end;
    local offset = 0;
		if graphical then
			offset = 70 * scale;
		end;
		d = D {'terminal_screen', 'simple_txt', {}, w = 569 * scale, h = h * scale, x = x * scale, y = y * scale, color = '#D0FFFF', color_link = '#abdece', color_alink = '#9bcebe', background=true, z=1000, font = 'fonts/vga.ttf', size=16 * scale, graphical = graphical, bgcolor='#000000', offset = offset};
	end;
	return d;
end;

function terminal_untheme(s)
	std.kh_leave_spaces = false;
	fmt.para = true;
end;

local function btn_anim_process(v)
	if v.animated == 1 then
		v.animated = true;
	elseif v.animated and v.frame_nr == 0 then
		v.animated = false;
    return true;
  else
    return true;
	end;
end;

local function shift_anim_process(v) 
	if v.frame_nr == 2 then
		v.animated = false;
  else
    return true;
	end;
end;

local keymapping = {
	['`'] = '~',
	['1'] = '!',
	['2'] = '@',
	['3'] = '#',
	['4'] = '$',
	['5'] = '%',
	['6'] = '^',
	['7'] = '&',
	['8'] = '*',
	['9'] = '(',
	['0'] = ')',
	['-'] = '_',
	['='] = '+',
	['q'] = 'Q', 
	['w'] = 'W', 
	['e'] = 'E',
	['r'] = 'R',
	['t'] = 'T',
	['y'] = 'Y',
	['u'] = 'U',
	['i'] = 'I', 
	['o'] = 'O',
	['p'] = 'P',
	['['] = '{',
	[']'] = '}',
	['\\'] = '|',
	['a'] = 'A',
	['s'] = 'S',
	['d'] = 'D',
	['f'] = 'F',
	['g'] = 'G',
	['h'] = 'H',
	['j'] = 'J',
	['k'] = 'K',
	['l'] = 'L',
	[';'] = ':',
	["'"] = '"',
	['z'] = 'Z',
	['x'] = 'X',
	['c'] = 'C',
	['v'] = 'V',
	['b'] = 'B',
	['n'] = 'N',
	['m'] = 'M',
	[','] = '.',
	['.'] = ',',
	['/'] = '?',
};

local reverse_keymapping = {
  ['>'] = '>';
  ['<'] = '<'
};
for k, v in pairs(keymapping) do
	reverse_keymapping[v] = k;
end;

local key_sounds = {
	'keyboard1', 'keyboard2', 'keyboard3', 'keyboard4', 'keyboard5'
}

local handled_keys = Set {
	'`', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=', 'backspace','shift',
	'q', 'w', 'e', 'r', 't', 'y', 'u','i', 'o', 'p', '[', ']', '\\', 
	'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', "'", 'return',
	'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', 
	'space', 'left', 'right', 'up', 'down', 'left shift'
}

local release_handled_keys = Set {
	'left shift'
}

local mapping_decor_key = {
	btn_backquote = '`';
	btn_1 = '1';
	btn_2 = '2';
	btn_3 = '3';
	btn_4 = '4';
	btn_5 = '5';
	btn_6 = '6';
	btn_7 = '7';
	btn_8 = '8';
	btn_9 = '9';
	btn_0 = '0';
	btn_minus = '-';
	btn_backspace = 'backspace';
	
	btn_q = 'q';
	btn_w = 'w';
	btn_e = 'e';
	btn_r = 'r';
	btn_t = 't';
	btn_y = 'y';
	btn_u = 'u';
	btn_i = 'i';
	btn_o = 'o';
	btn_p = 'p';
	btn_left_bracket = '[';
	btn_right_bracket = ']';
	btn_equal = '=';

	btn_a = 'a';
	btn_s = 's';
	btn_d = 'd';
	btn_f = 'f';
	btn_g = 'g';
	btn_h = 'h';
	btn_j = 'j';
	btn_k = 'k';
	btn_l = 'l';
	btn_semicolon = ';';
	btn_enter = 'return';


	btn_z = 'z';
	btn_x = 'x';
	btn_c = 'c';
	btn_v = 'v';
	btn_b = 'b';
	btn_n = 'n';
	btn_m = 'm';
	btn_comma = ',';
	btn_slash = '/';
	btn_backslash = '\\';
	btn_quote = '\'';
	btn_space = 'space';
};

local back_mapping_decor_key = {};

for k, v in pairs(mapping_decor_key) do
	back_mapping_decor_key[v] = k;
end;


function click:filter(press, btn, x, y, px, py)
  return not press
end


room {
	{ 
		_keyts = 0;
		_key = 0;
    _keypress = false;
    path = 'C:\\';
    entry = '';
    noinv = true;
    status = 'command';
    theme = 'terminal';
    format_percent = 0;
    mx = false;
    my = false;
	};
	
	nam = 'terminal';
	disp = '';
	nofollow = true;
	nofading = true;
  
	keysfilter = function(s, press, key)
		return handled_keys[key] ~= nil; -- ловим все нажатия
	end;
	
	query_formatter = function(s, path, entry)
		if s.status == 'command' then
			return s.path .. '>' .. s.entry .. '_';
		elseif s.status == 'crypt' or s.status == 'connect' then
			return translate_now(computer_locale, 'query_password') .. entry .. '_';
		elseif s.status == 'format' then
			return translate_now(computer_locale, 'query_type_yes') .. entry .. '_';
		elseif s.status == 'wait_format' then
			return translate_now(computer_locale, 'query_formatting') .. " " .. tostring(s.format_percent) .. '%';
		elseif s.status == 'format_done' then
		end;
		return '';
	end;
	
	command_preprocessor = function(s, command)
		if s.status == 'crypt' then
			s.status = 'command';
			s:push(translate_now(computer_locale, 'query_password') .. command);
			if command == password1 then
				s:push(password2);
				s:push('');
			else
				s:push(translate_now(computer_locale, 'invalid_password'));
				s:push('');
			end;
			s.entry = '';
			return true;
		elseif s.status == 'connect' then
			s.entry = '';
			s.status = 'command';
			s:push(translate_now(computer_locale, 'query_password') .. command);
			if command == password2 then
        if irradiated then
          snd.music('sound/bgm_ending_space_irradiated.ogg');
        else
          snd.music('sound/bgm_ending_space.ogg');
        end;
        if solarscale_with_me then
          walk('ending_solarscale_pre');
        else
          walk('ending_dissolving_tower_solarscale');
        end;
        
			else
				s:push(translate_now(computer_locale, 'invalid_password_2'));
			end;
			
			return true;
		elseif s.status == 'format' then
			s.status = 'format';
			s.entry = '';
			s:push(translate_now(computer_locale, 'query_type_yes') .. command);
			--s:push("Type 'Yes' to confirm:" .. command);
			if command:lower() == 'yes' then
				s.status = 'wait_format';
				timer:set(25);
			else
				s:push(translate_now(computer_locale, 'operation_cancelled'));
				s.status = 'command';
			end;
			
			return true;
		elseif s.status == 'wait_format' then
			s.entry = '';
			return true;
		end;
	end;
	timer = function(s)
		if s.status == 'wait_format' then
			if s.format_percent < 100 then
				s.format_percent = s.format_percent + 1;
				local d = getscreen(false);
				d:replace_line(s:query_formatter(s.path, s.entry));
			else
				s.status = 'format_done';
				s:push(translate_now(computer_locale, 'formatting_complete'));
			end;
			return false;
		elseif s.status == 'format_done' then
      if s.format_percent < 120 then
        s:push(translate_now(computer_locale, 'critical_failure'));
      end;
			
			s.format_percent = s.format_percent + 1;
			if s.format_percent > 150 then
				timer:stop();
				walk('terminal_boot_failure');
        return;
			end;
			return false;
		end;
		return false;
	end;
  puch_cmd = function(s, text)
    s:push_line(text);
  end;
  push = function(s, text)
    local lines = split(text, '^');
    
    for _, l in ipairs(lines) do
      s:push_line(l);
    end;
  end;
	push_line = function(s, line)
		local d = getscreen(false);
		d:replace_line(line);
		if s.query_formatter then
			d:insert_line(s:query_formatter(s.path, s.entry));
		else
			d:insert_line(s.path .. '>' .. s.entry .. '_');
		end;
	end;
	enter = function(s)
    s.path = 'C:\\';
    s.entry = '';
    
		s.history = {};
		s:process_directory_structure(s.directory_structure['C:']);
		s:push(translate_now(computer_locale, 'starting_tower'));
		s:push('');
		s:push(translate_now(computer_locale, 'tower_version'));
		s:push('');
		s:push(translate_now(computer_locale, 'type_help'));
		s:push('');

		s:makekeyboard();
    
    instead.autosave();
	end;
	makekeyboard = function(s)
		local width, height = diretheme.get_scaled_screen();
		
		local x = math.floor((width - 800) / 2);
		local y = height - 191;
    if height < 540 then
      y = y + 20;
    end;
		
		DS {'keyboard_bg', 'img', 'images/terminal/keyboard_bg.png', x=x - 116, y=y - 2, w=1036, h=191, z=110};
		-- Keyboard row 1
		-- D {'btn_backquote', 'img', 'images/terminal/keyboard/btn_backquote.png', x=x + 67, y=y, w=64, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
		DS {'btn_1', 'img', 'images/terminal/keyboard/btn_1.png', x=x + 119, y=y, w=58, h=46, frames=3, z=99, delay=50, process=btn_anim_process, click=true};
		DS {'btn_2', 'img', 'images/terminal/keyboard/btn_2.png', x=x + 167, y=y, w=55, h=46, frames=3, z=98, delay=50, process=btn_anim_process, click=true};
		DS {'btn_3', 'img', 'images/terminal/keyboard/btn_3.png', x=x + 215, y=y, w=51, h=46, frames=3, z=97, delay=50, process=btn_anim_process, click=true};
		DS {'btn_4', 'img', 'images/terminal/keyboard/btn_4.png', x=x + 260, y=y, w=49, h=46, frames=3, z=96, delay=50, process=btn_anim_process, click=true};
		DS {'btn_5', 'img', 'images/terminal/keyboard/btn_5.png', x=x + 307, y=y, w=44, h=46, frames=3, z=95, delay=50, process=btn_anim_process, click=true};
		DS {'btn_6', 'img', 'images/terminal/keyboard/btn_6.png', x=x + 351, y=y, w=47, h=46, frames=3, z=95, delay=50, process=btn_anim_process, click=true};
		DS {'btn_7', 'img', 'images/terminal/keyboard/btn_7.png', x=x + 398, y=y, w=48, h=46, frames=3, z=96, delay=50, process=btn_anim_process, click=true};
		DS {'btn_8', 'img', 'images/terminal/keyboard/btn_8.png', x=x + 444, y=y, w=48, h=46, frames=3, z=97, delay=50, process=btn_anim_process, click=true};
		DS {'btn_9', 'img', 'images/terminal/keyboard/btn_9.png', x=x + 486, y=y, w=51, h=46, frames=3, z=98, delay=50, process=btn_anim_process, click=true};
		DS {'btn_0', 'img', 'images/terminal/keyboard/btn_0.png', x=x + 529, y=y, w=55, h=46, frames=3, z=99, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_minus', 'img', 'images/terminal/keyboard/btn_minus.png', x=x + 574, y=y, w=58, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
		DS {'btn_backspace', 'img', 'images/terminal/keyboard/btn_backspace.png', x=x + 619, y=y, w=109, h=46, frames=3, z=101, delay=50, process=btn_anim_process, click=true};

		-- Keyboard row 2
		DS {'btn_q', 'img', 'images/terminal/keyboard/btn_q.png', x=x + 81, y=y + 26, w=61, h=46, frames=3, z=90, delay=50, process=btn_anim_process, click=true};
		DS {'btn_w', 'img', 'images/terminal/keyboard/btn_w.png', x=x + 130, y=y + 26, w=59, h=46, frames=3, z=89, delay=50, process=btn_anim_process, click=true};
		DS {'btn_e', 'img', 'images/terminal/keyboard/btn_e.png', x=x + 180, y=y + 26, w=57, h=46, frames=3, z=88, delay=50, process=btn_anim_process, click=true};
		DS {'btn_r', 'img', 'images/terminal/keyboard/btn_r.png', x=x + 230, y=y + 26, w=53, h=46, frames=3, z=87, delay=50, process=btn_anim_process, click=true};
		DS {'btn_t', 'img', 'images/terminal/keyboard/btn_t.png', x=x + 278, y=y + 26, w=50, h=46, frames=3, z=86, delay=50, process=btn_anim_process, click=true};
		DS {'btn_y', 'img', 'images/terminal/keyboard/btn_y.png', x=x + 327, y=y + 26, w=45, h=46, frames=3, z=85, delay=50, process=btn_anim_process, click=true};
		DS {'btn_u', 'img', 'images/terminal/keyboard/btn_u.png', x=x + 373, y=y + 26, w=52, h=46, frames=3, z=85, delay=50, process=btn_anim_process, click=true};
		DS {'btn_i', 'img', 'images/terminal/keyboard/btn_i.png', x=x + 423, y=y + 26, w=52, h=46, frames=3, z=86, delay=50, process=btn_anim_process, click=true};
		DS {'btn_o', 'img', 'images/terminal/keyboard/btn_o.png', x=x + 471, y=y + 26, w=50, h=46, frames=3, z=87, delay=50, process=btn_anim_process, click=true};
		DS {'btn_p', 'img', 'images/terminal/keyboard/btn_p.png', x=x + 514, y=y + 26, w=58, h=46, frames=3, z=88, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_left_bracket', 'img', 'images/terminal/keyboard/btn_left_bracket.png', x=x + 562, y=y + 26, w=60, h=46, frames=3, z=89, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_right_bracket', 'img', 'images/terminal/keyboard/btn_right_bracket.png', x=x + 610, y=y + 26, w=62, h=46, frames=3, z=90, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_equal', 'img', 'images/terminal/keyboard/btn_equal.png', x=x + 656, y=y + 26, w=65, h=46, frames=3, z=91, delay=50, process=btn_anim_process, click=true};
		
		-- Keyboard row 3
		DS {'btn_a', 'img', 'images/terminal/keyboard/btn_a.png', x=x + 81, y=y + 52, w=69, h=55, frames=3, z=80, delay=50, process=btn_anim_process, click=true};
		DS {'btn_s', 'img', 'images/terminal/keyboard/btn_s.png', x=x + 135, y=y + 52, w=67, h=55, frames=3, z=79, delay=50, process=btn_anim_process, click=true};
		DS {'btn_d', 'img', 'images/terminal/keyboard/btn_d.png', x=x + 191, y=y + 52, w=61, h=55, frames=3, z=78, delay=50, process=btn_anim_process, click=true};
		DS {'btn_f', 'img', 'images/terminal/keyboard/btn_f.png', x=x + 244, y=y + 52, w=58, h=55, frames=3, z=77, delay=50, process=btn_anim_process, click=true};
		DS {'btn_g', 'img', 'images/terminal/keyboard/btn_g.png', x=x + 296, y=y + 52, w=53, h=55, frames=3, z=76, delay=50, process=btn_anim_process, click=true};
		DS {'btn_h', 'img', 'images/terminal/keyboard/btn_h.png', x=x + 346, y=y + 52, w=55, h=55, frames=3, z=75, delay=50, process=btn_anim_process, click=true};
		DS {'btn_j', 'img', 'images/terminal/keyboard/btn_j.png', x=x + 401, y=y + 52, w=56, h=55, frames=3, z=76, delay=50, process=btn_anim_process, click=true};
		DS {'btn_k', 'img', 'images/terminal/keyboard/btn_k.png', x=x + 452, y=y + 52, w=55, h=55, frames=3, z=77, delay=50, process=btn_anim_process, click=true};
		DS {'btn_l', 'img', 'images/terminal/keyboard/btn_l.png', x=x + 500, y=y + 52, w=60, h=55, frames=3, z=78, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_semicolon', 'img', 'images/terminal/keyboard/btn_semicolon.png', x=x + 549, y=y + 52, w=65, h=55, frames=3, z=79, delay=50, process=btn_anim_process, click=true};
		DS {'btn_enter', 'img', 'images/terminal/keyboard/btn_enter.png', x=x + 599, y=y + 52, w=124, h=55, frames=3, z=80, delay=50, process=btn_anim_process, click=true};

		-- Keyboard row 4
		DS {'btn_z', 'img', 'images/terminal/keyboard/btn_z.png', x=x + 89, y=y + 86, w=74, h=59, frames=3, z=70, delay=50, process=btn_anim_process, click=true};
		DS {'btn_x', 'img', 'images/terminal/keyboard/btn_x.png', x=x + 147, y=y + 86, w=70, h=59, frames=3, z=69, delay=50, process=btn_anim_process, click=true};
		DS {'btn_c', 'img', 'images/terminal/keyboard/btn_c.png', x=x + 205, y=y + 86, w=64, h=59, frames=3, z=68, delay=50, process=btn_anim_process, click=true};
		DS {'btn_v', 'img', 'images/terminal/keyboard/btn_v.png', x=x + 261, y=y + 86, w=61, h=59, frames=3, z=67, delay=50, process=btn_anim_process, click=true};
		DS {'btn_b', 'img', 'images/terminal/keyboard/btn_b.png', x=x + 318, y=y + 86, w=53, h=59, frames=3, z=66, delay=50, process=btn_anim_process, click=true};
		DS {'btn_n', 'img', 'images/terminal/keyboard/btn_n.png', x=x + 372, y=y + 86, w=60, h=59, frames=3, z=66, delay=50, process=btn_anim_process, click=true};
		DS {'btn_m', 'img', 'images/terminal/keyboard/btn_m.png', x=x + 429, y=y + 86, w=61, h=59, frames=3, z=67, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_comma', 'img', 'images/terminal/keyboard/btn_comma.png', x=x + 483, y=y + 86, w=62, h=59, frames=3, z=68, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_slash', 'img', 'images/terminal/keyboard/btn_slash.png', x=x + 534, y=y + 86, w=69, h=59, frames=3, z=69, delay=50, process=btn_anim_process, click=true};
		-- DS {'btn_backslash', 'img', 'images/terminal/keyboard/btn_backslash.png', x=x + 589, y=y + 86, w=74, h=59, frames=3, z=70, delay=50, process=btn_anim_process, click=true};
		DS {'btn_quote', 'img', 'images/terminal/keyboard/btn_quote.png', x=x + 643, y=y + 86, w=78, h=59, frames=3, z=71, delay=50, process=btn_anim_process, click=true};

		local shift_frame = 0;
		if keys:state('shift') then
			shift_frame = 2;
		end;

		DS {'btn_shift', 'img', 'images/terminal/keyboard/btn_shift.png', x=x + 94, y=y + 125, w=139, h=64, frames=3, z=60, delay=50, process=shift_anim_process, click=true, frame_nr=shift_frame};
		DS {'btn_space', 'img', 'images/terminal/keyboard/btn_space.png', x=x + 222, y=y + 125, w=494, h=64, frames=3, z=59, delay=50, process=btn_anim_process, click=true};

		DS {'panel_1', 'img', 'images/terminal/keyboard/panel_1.png', x=x + 62, y=y + 37, w=677, h=16, z=92};
		DS {'panel_2', 'img', 'images/terminal/keyboard/panel_2.png', x=x + 77, y=y + 62, w=648, h=12,  z=81};
		DS {'panel_3', 'img', 'images/terminal/keyboard/panel_3.png', x=x + 78, y=y + 94, w=652, h=20, z=72};
		DS {'panel_4', 'img', 'images/terminal/keyboard/panel_4.png', x=x + 78, y=y + 131, w=652, h=23, z=61};
		DS {'panel_5', 'img', 'images/terminal/keyboard/panel_5.png', x=x, y=y + 181, w=800, h=10, z=51};

		if keys:state('shift') then
			DS {'btn_backquote', 'img', 'images/terminal/keyboard/btn_tilde.png', x=x + 67, y=y, w=64, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_minus', 'img', 'images/terminal/keyboard/btn_underscore.png', x=x + 574, y=y, w=58, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_left_bracket', 'img', 'images/terminal/keyboard/btn_left_curly.png', x=x + 562, y=y + 26, w=60, h=46, frames=3, z=89, delay=50, process=btn_anim_process, click=true};
			DS {'btn_right_bracket', 'img', 'images/terminal/keyboard/btn_right_curly.png', x=x + 610, y=y + 26, w=62, h=46, frames=3, z=90, delay=50, process=btn_anim_process, click=true};
			DS {'btn_equal', 'img', 'images/terminal/keyboard/btn_plus.png', x=x + 656, y=y + 26, w=65, h=46, frames=3, z=91, delay=50, process=btn_anim_process, click=true};
			DS {'btn_semicolon', 'img', 'images/terminal/keyboard/btn_colon.png', x=x + 549, y=y + 52, w=65, h=55, frames=3, z=79, delay=50, process=btn_anim_process, click=true};
			DS {'btn_comma', 'img', 'images/terminal/keyboard/btn_dot.png', x=x + 483, y=y + 86, w=62, h=59, frames=3, z=68, delay=50, process=btn_anim_process, click=true};
			DS {'btn_slash', 'img', 'images/terminal/keyboard/btn_question.png', x=x + 534, y=y + 86, w=69, h=59, frames=3, z=69, delay=50, process=btn_anim_process, click=true};
			DS {'btn_backslash', 'img', 'images/terminal/keyboard/btn_vertical.png', x=x + 589, y=y + 86, w=74, h=59, frames=3, z=70, delay=50, process=btn_anim_process, click=true};
		else
			DS {'btn_backquote', 'img', 'images/terminal/keyboard/btn_backquote.png', x=x + 67, y=y, w=64, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_minus', 'img', 'images/terminal/keyboard/btn_minus.png', x=x + 574, y=y, w=58, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_left_bracket', 'img', 'images/terminal/keyboard/btn_left_bracket.png', x=x + 562, y=y + 26, w=60, h=46, frames=3, z=89, delay=50, process=btn_anim_process, click=true};
			DS {'btn_right_bracket', 'img', 'images/terminal/keyboard/btn_right_bracket.png', x=x + 610, y=y + 26, w=62, h=46, frames=3, z=90, delay=50, process=btn_anim_process, click=true};
			DS {'btn_equal', 'img', 'images/terminal/keyboard/btn_equal.png', x=x + 656, y=y + 26, w=65, h=46, frames=3, z=91, delay=50, process=btn_anim_process, click=true};
			DS {'btn_semicolon', 'img', 'images/terminal/keyboard/btn_semicolon.png', x=x + 549, y=y + 52, w=65, h=55, frames=3, z=79, delay=50, process=btn_anim_process, click=true};
			DS {'btn_comma', 'img', 'images/terminal/keyboard/btn_comma.png', x=x + 483, y=y + 86, w=62, h=59, frames=3, z=68, delay=50, process=btn_anim_process, click=true};
			DS {'btn_slash', 'img', 'images/terminal/keyboard/btn_slash.png', x=x + 534, y=y + 86, w=69, h=59, frames=3, z=69, delay=50, process=btn_anim_process, click=true};
			DS {'btn_backslash', 'img', 'images/terminal/keyboard/btn_backslash.png', x=x + 589, y=y + 86, w=74, h=59, frames=3, z=70, delay=50, process=btn_anim_process, click=true};
		end;
	end;
	onexit = function(s)
		D {'keyboard_bg'};
		-- Keyboard row 1
		D {'btn_backquote'};
		D {'btn_1'};
		D {'btn_2'};
		D {'btn_3'};
		D {'btn_4'};
		D {'btn_5'};
		D {'btn_6'};
		D {'btn_7'};
		D {'btn_8'};
		D {'btn_9'};
		D {'btn_0'};
		D {'btn_minus'};
		D {'btn_backspace'};

		-- Keyboard row 2
		D {'btn_q'};
		D {'btn_w'};
		D {'btn_e'};
		D {'btn_r'};
		D {'btn_t'};
		D {'btn_y'};
		D {'btn_u'};
		D {'btn_i'};
		D {'btn_o'};
		D {'btn_p'};
		D {'btn_left_bracket'};
		D {'btn_right_bracket'};
		D {'btn_equal'};

		D {'btn_a'};
		D {'btn_s'};
		D {'btn_d'};
		D {'btn_f'};
		D {'btn_g'};
		D {'btn_h'};
		D {'btn_j'};
		D {'btn_k'};
		D {'btn_l'};
		D {'btn_semicolon'};
		D {'btn_enter'};

		D {'btn_z'};
		D {'btn_x'};
		D {'btn_c'};
		D {'btn_v'};
		D {'btn_b'};
		D {'btn_n'};
		D {'btn_m'};
		D {'btn_comma'};
		D {'btn_slash'};
		D {'btn_backslash'};
		D {'btn_quote'};

		D {'btn_shift'};
		D {'btn_space'};

		D {'panel_1'};
		D {'panel_2'};
		D {'panel_3'};
		D {'panel_4'};
		D {'panel_5'};
    D {'terminal_screen'};
	end;
	shift_buttons = function(s)
		local width, height = diretheme.get_scaled_screen();
		
		local x = math.floor((width - 800) / 2);
		local y = height - 191;
		if height < 540 then
      y = y + 20;
    end;

		if keys:state('shift') then
			DS {'btn_backquote', 'img', 'images/terminal/keyboard/btn_tilde.png', x=x + 67, y=y, w=64, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_minus', 'img', 'images/terminal/keyboard/btn_underscore.png', x=x + 574, y=y, w=58, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_left_bracket', 'img', 'images/terminal/keyboard/btn_left_curly.png', x=x + 562, y=y + 26, w=60, h=46, frames=3, z=89, delay=50, process=btn_anim_process, click=true};
			DS {'btn_right_bracket', 'img', 'images/terminal/keyboard/btn_right_curly.png', x=x + 610, y=y + 26, w=62, h=46, frames=3, z=90, delay=50, process=btn_anim_process, click=true};
			DS {'btn_equal', 'img', 'images/terminal/keyboard/btn_plus.png', x=x + 656, y=y + 26, w=65, h=46, frames=3, z=91, delay=50, process=btn_anim_process, click=true};
			DS {'btn_semicolon', 'img', 'images/terminal/keyboard/btn_colon.png', x=x + 549, y=y + 52, w=65, h=55, frames=3, z=79, delay=50, process=btn_anim_process, click=true};
			DS {'btn_comma', 'img', 'images/terminal/keyboard/btn_dot.png', x=x + 483, y=y + 86, w=62, h=59, frames=3, z=68, delay=50, process=btn_anim_process, click=true};
			DS {'btn_slash', 'img', 'images/terminal/keyboard/btn_question.png', x=x + 534, y=y + 86, w=69, h=59, frames=3, z=69, delay=50, process=btn_anim_process, click=true};
			DS {'btn_backslash', 'img', 'images/terminal/keyboard/btn_vertical.png', x=x + 589, y=y + 86, w=74, h=59, frames=3, z=70, delay=50, process=btn_anim_process, click=true};
		else
			DS {'btn_backquote', 'img', 'images/terminal/keyboard/btn_backquote.png', x=x + 67, y=y, w=64, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_minus', 'img', 'images/terminal/keyboard/btn_minus.png', x=x + 574, y=y, w=58, h=46, frames=3, z=100, delay=50, process=btn_anim_process, click=true};
			DS {'btn_left_bracket', 'img', 'images/terminal/keyboard/btn_left_bracket.png', x=x + 562, y=y + 26, w=60, h=46, frames=3, z=89, delay=50, process=btn_anim_process, click=true};
			DS {'btn_right_bracket', 'img', 'images/terminal/keyboard/btn_right_bracket.png', x=x + 610, y=y + 26, w=62, h=46, frames=3, z=90, delay=50, process=btn_anim_process, click=true};
			DS {'btn_equal', 'img', 'images/terminal/keyboard/btn_equal.png', x=x + 656, y=y + 26, w=65, h=46, frames=3, z=91, delay=50, process=btn_anim_process, click=true};
			DS {'btn_semicolon', 'img', 'images/terminal/keyboard/btn_semicolon.png', x=x + 549, y=y + 52, w=65, h=55, frames=3, z=79, delay=50, process=btn_anim_process, click=true};
			DS {'btn_comma', 'img', 'images/terminal/keyboard/btn_comma.png', x=x + 483, y=y + 86, w=62, h=59, frames=3, z=68, delay=50, process=btn_anim_process, click=true};
			DS {'btn_slash', 'img', 'images/terminal/keyboard/btn_slash.png', x=x + 534, y=y + 86, w=69, h=59, frames=3, z=69, delay=50, process=btn_anim_process, click=true};
			DS {'btn_backslash', 'img', 'images/terminal/keyboard/btn_backslash.png', x=x + 589, y=y + 86, w=74, h=59, frames=3, z=70, delay=50, process=btn_anim_process, click=true};
		end;
	end;
	onboot = function(s)
		s:process_directory_structure(s.directory_structure['C:']);
    
		s:makekeyboard();
    local d = getscreen(false);
    d:replace_line(s.path .. '>' .. s.entry .. '_');
	end;
	ondecor = function(s, name, press, x, y, btn) 
		if not press then
			if name == 'btn_shift' then
				if keys:state('shift') then
					keys:event(false, 'shift');
					D(name).frame_nr = 0;
          decor.dirty = true;
				else
					keys:event(true, 'shift');
					D(name).animated = 1;
					sounds[key_sounds[rnd(#key_sounds)]]:play();
				end;
				s:shift_buttons();
				return false;
			end;

			s:handledecor(true, mapping_decor_key[name]);
			return;
		end;

		return false;
	end;
	themeDispose = terminal_untheme;
	history = {};
	dsc = '';
  onkey = function(s, press, key)
    if not press and not release_handled_keys[key] then
			s._keyts = 0;
			return false;
		end;

		local ticks = instead.ticks();
		
		if s._key == key and ticks - s._keyts < 100 and s._keypress == press or key == 'return' and keys:state('alt') then
			return false;
		end;

		s._key = key;
		s._keyts = ticks;
    s._keypress = press;
    
    if key == 'left shift' then
			if press then
				if D('btn_shift').frame_nr ~= 2 then
					D('btn_shift').frame_nr = 2;
					sounds[key_sounds[rnd(#key_sounds)]]:play();
				end;
			else
				D('btn_shift').frame_nr = 0;
        D('btn_shift').animated = false;
			end;
      decor.dirty = true;
			s:shift_buttons();
			return;
		end;
    
    if not prefs.text_input or key == 'backspace' or key == 'left' or key == 'space' or key == 'return' then
      if keys:state('shift') and keymapping[key] then
				key = keymapping[key];
			end;
      
      s:oninput(key);
    end;
  end;
  handledecor = function(s, press, key)
    if key == 'left shift' then
			if press then
				if D('btn_shift').frame_nr ~= 2 then
					D('btn_shift').frame_nr = 2;
					sounds[key_sounds[rnd(#key_sounds)]]:play();
				end;
			else
				D('btn_shift').frame_nr = 0;
        D('btn_shift').animated = false;
			end;
      decor.dirty = true;
			s:shift_buttons();
			return;
		end;
    
    if keys:state('shift') and keymapping[key] then
      key = keymapping[key];
    end;
    
    s:oninput(key);
  end;
  ontext = function(s, char)
    if prefs.text_input then
      if (keymapping[char] or reverse_keymapping[char]) then
        s:oninput(char);
      elseif char ~= ' ' then
        sounds['error_beep']:play();
      end;
    end;
  end;
  oninput = function(s, key)
		sounds[key_sounds[rnd(#key_sounds)]]:play();
    
		local name = back_mapping_decor_key[key];
    if not name then
      local revkey = key;
      if reverse_keymapping[key] then
        revkey = reverse_keymapping[key];
      elseif keymapping[key] then
        revkey = keymapping[key];
      end;
      
      name = back_mapping_decor_key[revkey];
    end;
		
		if name then
			D(name).animated = 1;
			D(name).__delay = instead.ticks() - D(name).delay;
		end;
	
		if key == 'backspace' or key == 'left' then
			s.entry = s.entry:sub(1, #s.entry - 1);
		elseif key == 'right' or key == 'up' or key == 'down' then
		elseif key == 'space' then
			if #s.entry + #s.path + 3 > 63 then
				sounds['error_beep']:play();
				return;
			end;
			s.entry = s.entry .. ' ';
		elseif key == 'return' then
			local r = false;
			
			if s.command_preprocessor then
				r = s:command_preprocessor(s.entry);
			end;
			
			if not r then
				r = s:process_command(s.entry)
				if s:call_command(r[1], r[2]) then
          return;
        end;
				s.entry = '';
			end;
		else
			if #s.entry + #s.path + 3 > 63 then
				sounds['error_beep']:play();
				return;
			end;
		
			s.entry = s.entry .. key;
		end;
    if here() ^ s then
      local d = getscreen(false);
      if s.query_formatter then
        d:replace_line(s:query_formatter(s.path, s.entry));
      else
        d:replace_line(s.path .. '>' .. s.entry .. '_');
      end;
    end;
  end;
	process_command = function(s, str)
		s:puch_cmd(s.path .. '>' .. str);
		
		local rval = {}
		
		for i in string.gmatch(str, "%S+") do
			if (#rval > 0) then
				if rval[2] then
					rval[2] = rval[2] .. ' ' .. i;
				else
					rval[2] = i;
				end;
			else
				rval[1] = i;
			end;
			
		end;
		
		return rval;
	end;
	{
		directory_structure = {
			['C:'] = {
				['DOCS'] = {
					['PASSWORD.TXT'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push(translate_now(computer_locale, 'binary_data'));
						else
							s:push(translate_now(computer_locale, 'bad_command'));
						end;
					end;
					['CRYPT.COM'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push(translate_now(computer_locale, 'binary_data'));
						elseif command == '' then
							if not args then
								s:push(translate_now(computer_locale, 'crypt_usage'));
							else
								local new_args = {};
								for i in string.gmatch(args, "%S+") do
									if i and i ~= '' then
										new_args[#new_args + 1] = i;
									end;
								end;

								if #new_args > 2 then
									s:push(translate_now(computer_locale, 'crypt_usage'));
									return
								elseif #new_args < 2 then
                  s:push(translate_now(computer_locale, 'crypt_key_unset'));
									return;
								end;
								local name, path;
								name, path = s:walk_directory(new_args[1], true);
								
								if name ~= nil and path == 'C:\\DOCS\\PASSWORD.TXT' then
									name, path = s:walk_directory(new_args[2], true);
									if name ~= nil and path == 'C:\\DOCS\\KEY.TXT' then
										s:push(translate_now(computer_locale, 'crypt_key_locked'));
										s.status = 'crypt';
										return true;
									else
										s:push(translate_now(computer_locale, 'crypt_invalid_key'));
									end;
									
								else
									s:push(translate_now(computer_locale, 'crypt_unencrypted'));
								end;
							end;
						end;
					end;
					['CRYPT.CFG'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push('[MAIN]');
							s:push('ALGORITHM=AES');
							s:push('[RSA]');
							s:push('HARDWARE=FALSE');
							s:push(';KEY=KEY.TXT');
						else
							s:push(translate_now(computer_locale, 'bad_command'));
						end;
					end;
					['KEY.TXT'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push(translate_now(computer_locale, 'binary_data'));
						else
							s:push(translate_now(computer_locale, 'bad_command'));
						end;
					end;
				},
				['SYSTEM'] = {
					['CONNECT.EXE'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push(translate_now(computer_locale, 'binary_data'));
						elseif command == '' then
							s.status = 'connect';
							return true;
						end;
					end;
					['FORMAT.COM'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push(translate_now(computer_locale, 'binary_data'));
						elseif command == '' then
							if args and args:lower() == 'c:' then
								s.status = 'format';
								return true;
							elseif args and string.match(args, "^[A-z]:$") == args then
								s:push(translate_now(computer_locale, 'format_invalid_device'));
							else
                s:push(translate_now(computer_locale, 'format_usage'));
							end;
						end;
					end;
					['HELP.COM'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push(translate_now(computer_locale, 'binary_data'));
						elseif command == '' then
              s:push(translate_now(computer_locale, 'help_cd'));
              s:push(translate_now(computer_locale, 'help_dir'));
              s:push(translate_now(computer_locale, 'help_disk'));
              s:push(translate_now(computer_locale, 'help_connect'));
              s:push(translate_now(computer_locale, 'help_format'));
              s:push(translate_now(computer_locale, 'help_help'));
              s:push(translate_now(computer_locale, 'help_print'));
              s:push(translate_now(computer_locale, 'help_exit'));
              s:push(translate_now(computer_locale, 'help_com_exe'));
						end;
					end;
					['PRINT.COM'] = function(s, command, args)
						if command == 'PRINT.COM' then
							s:push(translate_now(computer_locale, 'binary_data'));
						elseif command == '' then
							if args then
								local cmd = s:walk_directory(args:upper(), true);
								if type(cmd) == 'function' then
									cmd(s, 'PRINT.COM', '')
								else
									s:push(translate_now(computer_locale, 'print_no_file'));
								end;
							else
								s:push(translate_now(computer_locale, 'print_usage'));
							end;
						end;
					end;
				},
				['AUTOEXEC.BAT'] = function(s, command, args)
					if command == 'PRINT.COM' then
						s:push('SET PATH=C:\\SYSTEM');
            s:push(translate_now(computer_locale, 'connect_comment'));
					elseif command == '' then
						s:push(translate_now(computer_locale, 'bad_command'));
					end;
				end;
				['CONFIG.SYS'] = function(s, command, args)
					if command == 'PRINT.COM' then
						s:push('DEVICE=C:\\SYSTEM\\HIMEM.SYS');
						s:push('DOS=HIGH,UMB');
						s:push('FILES=30');
						s:push('STACKS=0,0');
						s:push('BUFFERS=20');
						s:push('DEVICE=C:\\SYSTEM\\EMM386.EXE');
						s:push('DEVICE=C:\\SYSTEM\\TOWER.SYS');
					elseif command == '' then
						s:push(translate_now(computer_locale, 'bad_command'));
					end;
				end;
				['COMMAND.COM'] = function(s, command, args)
					if command == 'PRINT.COM' then
						s:push(translate_now(computer_locale, 'binary_data'));
					elseif command == '' then
						s:push(translate_now(computer_locale, 'command_already_started'));
					end;
				end;
				['HELLO.TXT'] = function(s, command, args)
					if command == 'PRINT.COM' then
            s:push(translate_now(computer_locale, 'hello_txt'));
					elseif command == '' then
						s:push(translate_now(computer_locale, 'bad_command'));
					end;
				end;
			}
		};
	};
	process_directory_structure = function(s, current, parent)
		current['.'] = current;
		current['..'] = parent;
		for k,v in pairs(current) do
			if k ~= '.' and k ~= '..' and type(v) == 'table' then
				s:process_directory_structure(v, current);
			end;
		end;
	end;
	walk_directory = function(s, path, no_replace)
		local location = s.directory_structure;
		
		if path:sub(1, 2):upper() ~= 'C:' then
			path = s.path .. path;
		end;
		
		local spath = {};
		
		for part in string.gmatch(path, "[^\\\\]+") do
			part = part:upper();
			if part == '.' then
			elseif part == '..' then
				spath[#spath] = nil;
			else
				spath[#spath + 1] = part;
			end;
			local oloc = location;
			location = location[part];
			if not no_replace then
				if location == nil then
					location = oloc[part .. '.COM'];
				end;
				if location == nil then
					location = oloc[part .. '.EXE'];
				end;
			end;

			if location == nil then
				return nil, nil
			end;
		end;
		return location, cat_list(spath, '\\');
	end;
	call_command = function(s, command, args)
		local r = false;
		if command == '' or not command then
			return
    end;
    
    command = command:lower();
    
		if command == 'dir' then
			local rval = {};
		
			local subdir = s:walk_directory(s.path);
			 local sort = function(s1, s2)
				 if type(subdir[s1]) == 'function' and type(subdir[s2]) == 'table' then
					 return false;
				 elseif type(subdir[s1]) == 'table' and type(subdir[s2]) == 'function' then
					 return true;
				 end;
				 return s1 < s2;
			end;
			
			for k, v in pairs(subdir) do
				rval[#rval + 1] = k;
			end;
			
			table.sort(rval, sort);
			
			for i, v in ipairs(rval) do
				s:push(v);
			end;
		elseif command == 'cd' then
			if args then
				args = args:upper();
				local dir, path = s:walk_directory(args)
				if type(dir) == 'table' then
					s.path = path .. '\\';
				else
          s:push(translate_now(computer_locale, 'cd_invalid_directory', {path=args}));
				end;
			else
        s:push(translate_now(computer_locale, 'cd_usage'));
			end;
		elseif command == 'ver' then
			s:push(translate_now(terminal_locale, 'tower_os_line_2'));
		elseif command == 'exit' then
			walk('tower_level_4_room_2');
      return true;
    elseif command == 'rm' and args and #args >= 2 and (args == '-rf . /' or args == '-rf /') then
      s:push(translate_now(terminal_locale, random_phrase({'unix_line_3', 'unix_line_4'})));
    elseif command == 'sudo' or command == 'su' or command == 'ls' or command == 'rm' then
      s:push(translate_now(terminal_locale, random_phrase({'unix_line_1', 'unix_line_2'})));
    elseif command == 'cat' then
      s:push(translate_now(terminal_locale, random_phrase({'cat_line_1', 'cat_line_2', 'cat_line_3'})));
    elseif command == 'more' or command == 'less' then
      s:push(translate_now(terminal_locale, 'more_line_1'));
    elseif command == 'head' or command == 'tail' then
      s:push(translate_now(terminal_locale, random_phrase({'head_line_1', 'head_line_2'})));
    elseif command == 'echo' then
      if args then
        s:push(args);
      else
        s:push('');
      end;
		elseif command == 'help' or command:lower() == 'help.com' then
			r = s.directory_structure['C:']['SYSTEM']['HELP.COM'](s, '', args);
		elseif command == 'print' or command:lower() == 'print.com' then
			r = s.directory_structure['C:']['SYSTEM']['PRINT.COM'](s, '', args);
		elseif command == 'format' or command:lower() == 'format.com' then
			r = s.directory_structure['C:']['SYSTEM']['FORMAT.COM'](s, '', args);
		elseif command == 'connect' or command:lower() == 'connect.exe' then
			r = s.directory_structure['C:']['SYSTEM']['CONNECT.EXE'](s, '', args);
		elseif #command == 2 and command:sub(2, 2) == ':' then
			if (s.directory_structure[command]) then
				s.path = command:upper() .. '\\';
			else
        s:push(translate_now(computer_locale, 'format_invalid_device'));
			end;
		else
			local cmd = s:walk_directory(command:upper())
			if type(cmd) == 'function' then
				r = cmd(s, '', args);
			else
        s:push(translate_now(computer_locale, 'bad_command'));
			end;
		end;
		if not r then
			s:push('');
		end;
	end;
}
